/*
  media-filesystem object query code
  tridge@samba.org, January 2001
  released under the Gnu GPL v2
*/

#include "mfs.h"

static struct {
	int fsid;
	void *buf;
	int size;
} loaded;

static void load_object(int fsid)
{
	if (fsid == loaded.fsid) return;

	if (loaded.buf) free(loaded.buf);

	loaded.size = mfs_fsid_size(fsid);
	loaded.buf = malloc(loaded.size);
	mfs_fsid_pread(fsid, loaded.buf, 0, loaded.size);
}

/* return the data portion of a part of an object */
void *query_part(int fsid, int subobj, char *name, int *len)
{
	void *ret = NULL;
	void callback(int fsid, struct mfs_subobj_header *obj,
		      struct mfs_attr_header *attr, void *data)
		{
			if (!attr) return;
			if (!(obj->flags && subobj == -1) && !(obj->id == subobj)) return;
			if (strcmp(schema_attrib(obj->obj_type, attr->attr), name)) return;
			*len = attr->len;
			ret = data;
			// fprintf(stderr, "len=%d ret=%*.*s\n", *len, *len, *len, ret);
		}

	if (mfs_fsid_type(fsid) != MFS_TYPE_OBJ) {
		fprintf(stderr,"%d is not an object\n", fsid);
		exit(1);
	}

	load_object(fsid);
	parse_object(fsid, loaded.buf, callback);
	return ret;
}

/* query a subobject path starting at fsid returning the data in the
   tail of the path */
void *query_path(int fsid, char *path, int *len)
{
	char *tok, *p;
	void *ret=NULL;
	int subobj = -1;
	
	path = strdup(path);
	tok = strtok_r(path,"/", &p);

	while (tok) {
		// fprintf(stderr, "%d/%d %s\n", fsid, subobj, tok);
		ret = query_part(fsid, subobj, tok, len);
		if (!ret) return NULL;
		tok = strtok_r(NULL,"/", &p);
		if (tok) {
			struct mfs_obj_attr *objattr = ret;
			fsid = ntohl(objattr->fsid);
			subobj = ntohl(objattr->subobj);
		}
	}
	free(path);
	return ret;
}

char *query_string(int fsid, char *path)
{
	int len;
	char *p = query_path(fsid, path, &len);
	// fprintf(stderr, "str=[%s]\n", p);
	return p;
}

int query_int(int fsid, char *path)
{
	int len;
	int *p = query_path(fsid, path, &len);
	if (!p) return -1;
	return ntohl(*p);
}

struct mfs_obj_attr query_object(int fsid, char *path)
{
	int len;
	struct mfs_obj_attr ret = {-1, -1};
	struct mfs_obj_attr *p = query_path(fsid, path, &len);
	if (!p) return ret;
	ret = *p;
	ret.fsid = ntohl(ret.fsid);
	ret.subobj = ntohl(ret.subobj);
	return ret;
}

void query_streams(char *path)
{
	struct mfs_dirent *dir;
	u32 count, i;
	dir = mfs_dir(mfs_resolve(path), &count);
	for (i=0;i<count;i++) {
		fprintf(stderr, "%d: ",
		       query_int(dir[i].fsid, "Part/File"));
		fprintf(stderr, "%s\n", 
		       query_string(dir[i].fsid, "Showing/Program/Title"));
	}
	if (dir) mfs_dir_free(dir);
}
